import logging
from decimal import Decimal

from django.conf import settings
from django.test import TestCase
from djmoney.money import Money

from order.enum.status import OrderStatus
from order.factories.order import OrderFactory
from product.factories.product import ProductFactory

logger = logging.getLogger(__name__)


class OrderModelTestCase(TestCase):
    def setUp(self):
        self.order = OrderFactory(
            pay_way=None,
            email="test@example.com",
            first_name="John",
            last_name="Doe",
            street="123 Main St",
            street_number="Apt 4",
            city="New York",
            zipcode="10001",
            phone="123-456-7890",
            status=OrderStatus.PENDING,
            shipping_price=Money("10.00", settings.DEFAULT_CURRENCY),
            paid_amount=Money("0", settings.DEFAULT_CURRENCY),
        )

        products = ProductFactory.create_batch(2, num_images=0, num_reviews=0)

        self.product_1 = products[0]
        self.product_2 = products[1]

        self.order.items.create(
            product_id=self.product_1.id,
            price=Decimal("50.00"),
            quantity=2,
        )
        self.order.items.create(
            product_id=self.product_2.id,
            price=Decimal("30.00"),
            quantity=3,
        )

    def test_total_price_items_with_items(self):
        expected_total_price = sum(
            item.total_price.amount for item in self.order.items.all()
        )
        self.assertEqual(
            self.order.total_price_items.amount, expected_total_price
        )

    def test_total_price_extra_without_pay_way(self):
        self.assertEqual(
            self.order.total_price_extra, self.order.shipping_price
        )

    def test_full_address(self):
        expected_full_address = f"{self.order.street} {self.order.street_number}, {self.order.zipcode} {self.order.city}"
        self.assertEqual(self.order.full_address, expected_full_address)

    def test_fields(self):
        self.assertEqual(self.order.email, "test@example.com")
        self.assertEqual(self.order.first_name, "John")
        self.assertEqual(self.order.last_name, "Doe")
        self.assertEqual(self.order.street, "123 Main St")
        self.assertEqual(self.order.street_number, "Apt 4")
        self.assertEqual(self.order.city, "New York")
        self.assertEqual(self.order.zipcode, "10001")
        self.assertEqual(self.order.phone, "123-456-7890")
        self.assertEqual(self.order.status, OrderStatus.PENDING)
        self.assertEqual(
            self.order.shipping_price,
            Money("10.00", settings.DEFAULT_CURRENCY),
        )
        self.assertEqual(
            self.order.paid_amount, Money("0", settings.DEFAULT_CURRENCY)
        )

    def test_str_representation(self):
        self.assertEqual(str(self.order), f"Order {self.order.id} - John Doe")



from django.urls import path
from rest_framework.urlpatterns import format_suffix_patterns

from cart.views.cart import CartViewSet
from cart.views.item import CartItemViewSet

urlpatterns = [
    path(
        "cart",
        CartViewSet.as_view(
            {
                "get": "retrieve",
                "put": "update",
                "patch": "partial_update",
                "delete": "destroy",
            }
        ),
        name="cart-detail",
    ),
    path(
        "cart/list",
        CartViewSet.as_view({"get": "list"}),
        name="cart-list",
    ),
    path(
        "cart/item",
        CartItemViewSet.as_view({"get": "list", "post": "create"}),
        name="cart-item-list",
    ),
    path(
        "cart/item/<int:pk>",
        CartItemViewSet.as_view(
            {
                "get": "retrieve",
                "put": "update",
                "patch": "partial_update",
                "delete": "destroy",
            }
        ),
        name="cart-item-detail",
    ),
]

urlpatterns = format_suffix_patterns(urlpatterns)



from blog.models.post import BlogPost
from blog.strategies.related_posts_strategy import RelatedPostsStrategy


class WeightedRelatedPostsStrategy(RelatedPostsStrategy):
    def __init__(
        self,
        strategies_with_weights: list[tuple[RelatedPostsStrategy, float]],
        limit: int = 8,
    ):
        self.strategies_with_weights = strategies_with_weights
        self.limit = limit

    def get_related_posts(self, post: BlogPost):
        collected_posts = []
        collected_post_ids = set()
        remaining_limit = self.limit

        for strategy, weight in self.strategies_with_weights:
            num_posts = int(weight * self.limit)
            num_posts = min(num_posts, remaining_limit)
            if num_posts <= 0:
                continue

            posts = strategy.get_related_posts(post).exclude(
                pk__in=collected_post_ids
            )[:num_posts]
            collected_posts.extend(posts)
            collected_post_ids.update(posts.values_list("pk", flat=True))
            remaining_limit -= len(posts)
            if remaining_limit <= 0:
                break

        if remaining_limit > 0:
            for strategy, _weight in self.strategies_with_weights:
                posts = strategy.get_related_posts(post).exclude(
                    pk__in=collected_post_ids
                )[:remaining_limit]
                for p in posts:
                    if p.pk not in collected_post_ids:
                        collected_posts.append(p)
                        collected_post_ids.add(p.pk)
                        remaining_limit -= 1
                        if remaining_limit <= 0:
                            break
                if remaining_limit <= 0:
                    break

        return BlogPost.objects.filter(
            pk__in=[p.pk for p in collected_posts]
        ).order_by("-published_at")



from decimal import Decimal
from unittest import TestCase, mock

from django.conf import settings
from djmoney.money import Money

from order.enum.status import PaymentStatus
from order.payment import (
    PayPalPaymentProvider,
    StripePaymentProvider,
    get_payment_provider,
)


class PaymentModuleTestCase(TestCase):
    def test_payment_status_enum(self):
        self.assertEqual(PaymentStatus.PENDING.value, "PENDING")
        self.assertEqual(PaymentStatus.PROCESSING.value, "PROCESSING")
        self.assertEqual(PaymentStatus.COMPLETED.value, "COMPLETED")
        self.assertEqual(PaymentStatus.FAILED.value, "FAILED")
        self.assertEqual(PaymentStatus.REFUNDED.value, "REFUNDED")
        self.assertEqual(
            PaymentStatus.PARTIALLY_REFUNDED.value, "PARTIALLY_REFUNDED"
        )
        self.assertEqual(PaymentStatus.CANCELED.value, "CANCELED")

    @mock.patch("order.payment.settings")
    def test_stripe_payment_provider_init(self, mock_settings):
        mock_settings.STRIPE_API_KEY = "test_api_key"
        mock_settings.STRIPE_WEBHOOK_SECRET = "test_webhook_secret"

        provider = StripePaymentProvider()

        self.assertEqual(provider.api_key, "test_api_key")
        self.assertEqual(provider.webhook_secret, "test_webhook_secret")

    @mock.patch("order.payment.settings")
    def test_paypal_payment_provider_init(self, mock_settings):
        mock_settings.PAYPAL_CLIENT_ID = "test_client_id"
        mock_settings.PAYPAL_CLIENT_SECRET = "test_client_secret"

        provider = PayPalPaymentProvider()

        self.assertEqual(provider.client_id, "test_client_id")
        self.assertEqual(provider.client_secret, "test_client_secret")

    @mock.patch("order.payment.logger")
    def test_stripe_process_payment(self, mock_logger):
        provider = StripePaymentProvider()
        amount = Money(
            amount=Decimal("100.00"), currency=settings.DEFAULT_CURRENCY
        )
        order_id = "test_order_id"

        success, payment_data = provider.process_payment(amount, order_id)

        self.assertTrue(success)
        self.assertEqual(payment_data["payment_id"], f"pi_{order_id}_mock")
        self.assertEqual(payment_data["status"], PaymentStatus.COMPLETED)
        self.assertEqual(payment_data["amount"], str(amount.amount))
        self.assertEqual(payment_data["currency"], amount.currency)
        self.assertEqual(payment_data["provider"], "stripe")

        mock_logger.info.assert_called_once()

    @mock.patch("order.payment.logger")
    def test_paypal_process_payment(self, mock_logger):
        provider = PayPalPaymentProvider()
        amount = Money(
            amount=Decimal("100.00"), currency=settings.DEFAULT_CURRENCY
        )
        order_id = "test_order_id"

        success, payment_data = provider.process_payment(amount, order_id)

        self.assertTrue(success)
        self.assertEqual(payment_data["payment_id"], f"PP_{order_id}_mock")
        self.assertEqual(payment_data["status"], PaymentStatus.COMPLETED)
        self.assertEqual(payment_data["amount"], str(amount.amount))
        self.assertEqual(payment_data["currency"], amount.currency)
        self.assertEqual(payment_data["provider"], "paypal")

        mock_logger.info.assert_called_once()

    @mock.patch("order.payment.logger")
    def test_stripe_refund_payment(self, mock_logger):
        provider = StripePaymentProvider()
        payment_id = "test_payment_id"
        amount = Money(
            amount=Decimal("50.00"), currency=settings.DEFAULT_CURRENCY
        )

        success, refund_data = provider.refund_payment(payment_id, amount)

        self.assertTrue(success)
        self.assertEqual(refund_data["refund_id"], f"re_{payment_id}_mock")
        self.assertEqual(refund_data["status"], PaymentStatus.REFUNDED)
        self.assertEqual(refund_data["amount"], str(amount.amount))
        self.assertEqual(refund_data["payment_id"], payment_id)

        mock_logger.info.assert_called_once()

    @mock.patch("order.payment.logger")
    def test_stripe_get_payment_status(self, mock_logger):
        provider = StripePaymentProvider()
        payment_id = "test_payment_id"

        status, status_data = provider.get_payment_status(payment_id)

        self.assertEqual(status, PaymentStatus.COMPLETED)
        self.assertEqual(status_data["payment_id"], payment_id)
        self.assertEqual(status_data["raw_status"], "succeeded")
        self.assertEqual(status_data["provider"], "stripe")

        mock_logger.info.assert_called_once()

    def test_get_payment_provider(self):
        provider = get_payment_provider("stripe")
        self.assertIsInstance(provider, StripePaymentProvider)

        provider = get_payment_provider("paypal")
        self.assertIsInstance(provider, PayPalPaymentProvider)

        with self.assertRaises(ValueError):
            get_payment_provider("invalid_provider")



from unittest import mock

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.sessions.middleware import SessionMiddleware
from django.test import TestCase
from djmoney.money import Money
from rest_framework import status
from rest_framework.test import APIClient, APIRequestFactory, force_authenticate

from order.enum.status import PaymentStatus
from order.factories.order import OrderFactory
from order.views.payment import OrderPaymentViewSet
from pay_way.factories import PayWayFactory

User = get_user_model()


def get_response(request):
    return None


def add_session_to_request(request):
    middleware = SessionMiddleware(get_response)
    middleware.process_request(request)
    request.session.save()
    return request


class OrderPaymentViewSetTestCase(TestCase):
    def setUp(self):
        self.client = APIClient()
        self.factory = APIRequestFactory()

        self.admin_user = User.objects.create_superuser(
            username="admin", email="admin@example.com", password="password"
        )

        self.user = User.objects.create_user(
            username="user", email="user@example.com", password="password"
        )

        self.stripe_pay_way = PayWayFactory(
            active=True,
            provider_code="stripe",
            is_online_payment=True,
            requires_confirmation=False,
        )

        self.bank_transfer_pay_way = PayWayFactory(
            active=True,
            provider_code="",
            is_online_payment=False,
            requires_confirmation=True,
        )

        self.user_order = OrderFactory(
            user=self.user,
            pay_way=None,
            payment_status=PaymentStatus.PENDING,
            payment_id="",
            payment_method="",
            status="PENDING",
            paid_amount=Money(0, settings.DEFAULT_CURRENCY),
        )

        self.anon_order = OrderFactory(
            user=None,
            pay_way=None,
            payment_status=PaymentStatus.PENDING,
            payment_id="",
            payment_method="",
            status="PENDING",
            paid_amount=Money(0, settings.DEFAULT_CURRENCY),
        )

        self.paid_order = OrderFactory(
            user=self.user,
            pay_way=self.stripe_pay_way,
            payment_status=PaymentStatus.COMPLETED,
            payment_id="PAID_123",
            payment_method="Stripe",
            status="COMPLETED",
        )
        self.paid_order.paid_amount = self.paid_order.total_price
        self.paid_order.save()

    @mock.patch(
        "order.models.order.Order.is_paid", new_callable=mock.PropertyMock
    )
    def test_process_payment_authenticated_user(self, mock_is_paid):
        mock_is_paid.return_value = False

        self.client.force_authenticate(user=self.user)

        view = OrderPaymentViewSet.as_view({"post": "process_payment"})
        request = self.factory.post(
            f"/api/v1/order/{self.user_order.pk}/process_payment",
            {
                "pay_way_id": self.stripe_pay_way.id,
                "payment_data": {"token": "test_token"},
            },
            format="json",
        )
        force_authenticate(request, user=self.user)

        with mock.patch(
            "order.views.payment.PayWayService.process_payment"
        ) as mock_process_payment:
            mock_process_payment.return_value = (
                True,
                {
                    "payment_id": "TEST_PAYMENT_123",
                    "status": PaymentStatus.COMPLETED,
                    "amount": "100.00",
                    "currency": settings.DEFAULT_CURRENCY,
                    "provider": "stripe",
                },
            )

            response = view(request, pk=self.user_order.pk)

            self.assertEqual(response.status_code, status.HTTP_200_OK)
            self.assertEqual(response.data["order_id"], self.user_order.id)
            self.assertIn("payment_status", response.data)
            self.assertIn("payment_id", response.data)

    @mock.patch(
        "order.models.order.Order.is_paid", new_callable=mock.PropertyMock
    )
    def test_process_payment_anonymous_user(self, mock_is_paid):
        mock_is_paid.return_value = False

        request = self.factory.post(
            f"/api/v1/order/{self.anon_order.pk}/process_payment",
            {"pay_way_id": self.bank_transfer_pay_way.id, "payment_data": {}},
            format="json",
        )

        add_session_to_request(request)

        view = OrderPaymentViewSet.as_view({"post": "process_payment"})

        with mock.patch(
            "order.views.payment.PayWayService.process_payment"
        ) as mock_process_payment:
            mock_process_payment.return_value = (
                True,
                {
                    "payment_id": f"OFFLINE_{self.anon_order.id}",
                    "status": PaymentStatus.PENDING,
                    "amount": "100.00",
                    "currency": settings.DEFAULT_CURRENCY,
                    "provider": "offline",
                },
            )

            response = view(request, pk=self.anon_order.pk)

            self.assertEqual(response.status_code, status.HTTP_200_OK)
            mock_process_payment.assert_called_once()

            self.assertTrue(
                request.session.get(f"order_{self.anon_order.uuid}")
            )

    @mock.patch(
        "order.models.order.Order.is_paid", new_callable=mock.PropertyMock
    )
    def test_process_payment_unauthorized_user(self, mock_is_paid):
        mock_is_paid.return_value = False

        regular_user = User.objects.create_user(
            username="another_user",
            email="another@example.com",
            password="password",
        )

        view = OrderPaymentViewSet.as_view({"post": "process_payment"})
        request = self.factory.post(
            f"/api/v1/order/{self.user_order.pk}/process_payment",
            {
                "pay_way_id": self.stripe_pay_way.id,
                "payment_data": {"token": "test_token"},
            },
            format="json",
        )
        force_authenticate(request, user=regular_user)
        response = view(request, pk=self.user_order.pk)

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        request = self.factory.post(
            f"/api/v1/order/{self.user_order.pk}/process_payment",
            {
                "pay_way_id": self.stripe_pay_way.id,
                "payment_data": {"token": "test_token"},
            },
            format="json",
        )
        force_authenticate(request, user=self.admin_user)

        with mock.patch(
            "order.views.payment.PayWayService.process_payment"
        ) as mock_process:
            mock_process.return_value = (True, {"payment_id": "test"})
            response = view(request, pk=self.user_order.pk)
            self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_process_payment_already_paid(self):
        view = OrderPaymentViewSet.as_view({"post": "process_payment"})
        request = self.factory.post(
            f"/api/v1/order/{self.paid_order.pk}/process_payment",
            {
                "pay_way_id": self.stripe_pay_way.id,
                "payment_data": {"token": "test_token"},
            },
            format="json",
        )
        force_authenticate(request, user=self.user)
        response = view(request, pk=self.paid_order.pk)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn("already been paid", response.data["detail"])

    @mock.patch("order.views.payment.PayWayService.check_payment_status")
    def test_check_payment_status(self, mock_check_status):
        mock_check_status.return_value = (
            PaymentStatus.COMPLETED,
            {
                "payment_id": "TEST_PAYMENT_123",
                "raw_status": "succeeded",
                "provider": "stripe",
            },
        )

        order = self.user_order
        order.payment_id = "TEST_PAYMENT_123"
        order.pay_way = self.stripe_pay_way
        order.save()

        view = OrderPaymentViewSet.as_view({"get": "check_payment_status"})
        request = self.factory.get(f"/api/v1/order/{order.pk}/payment_status")
        force_authenticate(request, user=self.user)
        response = view(request, pk=order.pk)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        mock_check_status.assert_called_once()

        self.assertEqual(response.data["order_id"], order.id)
        self.assertEqual(
            response.data["payment_status"], PaymentStatus.COMPLETED
        )

    def test_check_payment_status_no_payment(self):
        view = OrderPaymentViewSet.as_view({"get": "check_payment_status"})
        request = self.factory.get(
            f"/api/v1/order/{self.user_order.pk}/payment_status"
        )
        force_authenticate(request, user=self.user)
        response = view(request, pk=self.user_order.pk)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn("no payment method", response.data["detail"])

        self.user_order.pay_way = self.stripe_pay_way
        self.user_order.save()

        request = self.factory.get(
            f"/api/v1/order/{self.user_order.pk}/payment_status"
        )
        force_authenticate(request, user=self.user)
        response = view(request, pk=self.user_order.pk)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn(
            "This order has no associated payment", response.data["detail"]
        )

    @mock.patch("order.views.payment.PayWayService.refund_payment")
    @mock.patch(
        "order.models.order.Order.is_paid", new_callable=mock.PropertyMock
    )
    def test_refund_payment_admin_user(self, mock_is_paid, mock_refund):
        mock_is_paid.return_value = True

        self.paid_order.refresh_from_db()
        if not self.paid_order.pay_way:
            self.paid_order.pay_way = self.stripe_pay_way
            self.paid_order.save()

        mock_refund.return_value = (
            True,
            {
                "refund_id": "REFUND_123",
                "status": PaymentStatus.REFUNDED,
                "amount": "full refund",
                "payment_id": "PAID_123",
            },
        )

        view = OrderPaymentViewSet.as_view({"post": "refund_payment"})
        request = self.factory.post(
            f"/api/v1/order/{self.paid_order.pk}/refund"
        )
        force_authenticate(request, user=self.admin_user)
        response = view(request, pk=self.paid_order.pk)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        mock_refund.assert_called_once()

        self.assertEqual(response.data["order_id"], self.paid_order.id)
        self.assertIn("payment_status", response.data)
        self.assertEqual(response.data["refund_id"], "REFUND_123")
        self.assertIn("refund_details", response.data)

    def test_refund_payment_non_admin_user(self):
        self.paid_order.refresh_from_db()
        if not self.paid_order.pay_way:
            self.paid_order.pay_way = self.stripe_pay_way
            self.paid_order.save()

        view = OrderPaymentViewSet.as_view({"post": "refund_payment"})
        request = self.factory.post(
            f"/api/v1/order/{self.paid_order.pk}/refund"
        )
        force_authenticate(request, user=self.user)
        response = view(request, pk=self.paid_order.pk)

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @mock.patch("order.views.payment.PayWayService.refund_payment")
    @mock.patch(
        "order.models.order.Order.is_paid", new_callable=mock.PropertyMock
    )
    def test_refund_payment_partial(self, mock_is_paid, mock_refund):
        mock_is_paid.return_value = True

        self.paid_order.refresh_from_db()
        if not self.paid_order.pay_way:
            self.paid_order.pay_way = self.stripe_pay_way
            self.paid_order.save()

        mock_refund.return_value = (
            True,
            {
                "refund_id": "REFUND_123",
                "status": PaymentStatus.PARTIALLY_REFUNDED,
                "amount": "50.00",
                "currency": settings.DEFAULT_CURRENCY,
                "payment_id": "PAID_123",
            },
        )

        view = OrderPaymentViewSet.as_view({"post": "refund_payment"})
        request = self.factory.post(
            f"/api/v1/order/{self.paid_order.pk}/refund",
            {"amount": 50.00, "currency": settings.DEFAULT_CURRENCY},
            format="json",
        )
        force_authenticate(request, user=self.admin_user)
        response = view(request, pk=self.paid_order.pk)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        amount_arg = mock_refund.call_args[1]["amount"]
        self.assertEqual(amount_arg.amount, 50.00)
        self.assertEqual(str(amount_arg.currency), settings.DEFAULT_CURRENCY)



from __future__ import annotations

from urllib.parse import unquote

from django.utils.translation import gettext_lazy as _
from drf_spectacular.utils import OpenApiParameter, extend_schema
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response

from blog.models.post import BlogPostTranslation
from core.api.serializers import ErrorResponseSerializer
from product.models.product import ProductTranslation
from search.serializers import (
    BlogPostMeiliSearchResponseSerializer,
    BlogPostTranslationSerializer,
    ProductMeiliSearchResponseSerializer,
    ProductTranslationSerializer,
)


@extend_schema(
    summary=_("Search blog posts"),
    description=_(
        "Search blog posts using MeiliSearch. Provides full-text search with "
        "highlighting, ranking, and faceting capabilities."
    ),
    tags=["Search"],
    responses={
        200: BlogPostMeiliSearchResponseSerializer,
        400: ErrorResponseSerializer,
    },
    parameters=[
        OpenApiParameter(
            name="query",
            type=str,
            location=OpenApiParameter.QUERY,
            description="Search query string",
            required=True,
        ),
        OpenApiParameter(
            name="limit",
            type=int,
            location=OpenApiParameter.QUERY,
            description="Maximum number of results to return",
            required=False,
            default=10,
        ),
        OpenApiParameter(
            name="offset",
            type=int,
            location=OpenApiParameter.QUERY,
            description="Number of results to skip",
            required=False,
            default=0,
        ),
    ],
)
@api_view(["GET"])
def blog_post_meili_search(request):
    query = request.query_params.get("query")
    if not query:
        raise ValidationError({"error": "A search query is required."})

    limit = int(request.query_params.get("limit", 10))
    offset = int(request.query_params.get("offset", 0))

    decoded_query = unquote(query)

    enriched_results = BlogPostTranslation.meilisearch.paginate(
        limit=limit, offset=offset
    ).search(q=decoded_query)

    serialized_data = []
    for result in enriched_results["results"]:
        obj = result["object"]
        context = {
            "_formatted": result.get("_formatted", {}),
            "_matchesPosition": result.get("_matchesPosition", {}),
            "_rankingScore": result.get("_rankingScore", None),
        }
        obj_data = BlogPostTranslationSerializer(obj, context=context).data
        serialized_data.append(obj_data)

    return Response(
        {
            "limit": limit,
            "offset": offset,
            "estimated_total_hits": enriched_results["estimated_total_hits"],
            "results": serialized_data,
        },
        status=status.HTTP_200_OK,
    )


@extend_schema(
    summary=_("Search products"),
    description=_(
        "Search products using MeiliSearch. Provides full-text search with "
        "highlighting, ranking, and faceting capabilities."
    ),
    tags=["Search"],
    responses={
        200: ProductMeiliSearchResponseSerializer,
        400: ErrorResponseSerializer,
    },
    parameters=[
        OpenApiParameter(
            name="query",
            type=str,
            location=OpenApiParameter.QUERY,
            description="Search query string",
            required=True,
        ),
        OpenApiParameter(
            name="limit",
            type=int,
            location=OpenApiParameter.QUERY,
            description="Maximum number of results to return",
            required=False,
            default=10,
        ),
        OpenApiParameter(
            name="offset",
            type=int,
            location=OpenApiParameter.QUERY,
            description="Number of results to skip",
            required=False,
            default=0,
        ),
    ],
)
@api_view(["GET"])
def product_meili_search(request):
    query = request.query_params.get("query")
    if not query:
        raise ValidationError({"error": "A search query is required."})

    limit = int(request.query_params.get("limit", 10))
    offset = int(request.query_params.get("offset", 0))

    decoded_query = unquote(query)

    enriched_results = ProductTranslation.meilisearch.paginate(
        limit=limit, offset=offset
    ).search(q=decoded_query)

    serialized_data = []
    for result in enriched_results["results"]:
        obj = result["object"]
        context = {
            "_formatted": result.get("_formatted", {}),
            "_matchesPosition": result.get("_matchesPosition", {}),
            "_rankingScore": result.get("_rankingScore", None),
        }
        obj_data = ProductTranslationSerializer(obj, context=context).data
        serialized_data.append(obj_data)

    return Response(
        {
            "limit": limit,
            "offset": offset,
            "estimated_total_hits": enriched_results["estimated_total_hits"],
            "results": serialized_data,
        },
        status=status.HTTP_200_OK,
    )



from .category import ProductCategoryFactory
from .category_image import ProductCategoryImageFactory
from .favourite import ProductFavouriteFactory
from .image import ProductImageFactory
from .product import ProductFactory
from .review import ProductReviewFactory

__all__ = [
    "ProductCategoryFactory",
    "ProductCategoryImageFactory",
    "ProductFactory",
    "ProductFavouriteFactory",
    "ProductImageFactory",
    "ProductReviewFactory",
]



# Generated by Django 5.0.7 on 2024-07-21 00:11
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("tag", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="tag",
            options={
                "ordering": ["sort_order"],
                "verbose_name": "Tag",
                "verbose_name_plural": "Tags",
            },
        ),
        migrations.AlterModelOptions(
            name="tagtranslation",
            options={
                "default_permissions": (),
                "managed": True,
                "verbose_name": "Tag Translation",
            },
        ),
    ]



from datetime import timedelta
from django.urls import reverse
from django.utils import timezone
from django.test import TransactionTestCase
from rest_framework.test import APIClient
import pytest

from country.factories import CountryFactory
from region.factories import RegionFactory
from user.factories.account import UserAccountFactory
from user.models.account import UserAccount


@pytest.mark.django_db(transaction=True)
class UserAccountFilterTest(TransactionTestCase):
    def setUp(self):
        UserAccount.objects.all().delete()

        self.client = APIClient()

        self.auth_user = UserAccountFactory(
            email="auth@example.com",
            is_superuser=True,
            is_staff=True,
        )
        self.client.force_authenticate(user=self.auth_user)

        self.country_gr = CountryFactory(alpha_2="GR", name="Greece")
        self.country_de = CountryFactory(alpha_2="DE", name="Germany")
        self.region_athens = RegionFactory(alpha="ATH", country=self.country_gr)
        self.region_berlin = RegionFactory(alpha="BER", country=self.country_de)

        self.now = timezone.now()

        self.admin_user = UserAccountFactory(
            email="admin@example.com",
            username="admin_user",
            first_name="Admin",
            last_name="User",
            is_staff=True,
            is_active=True,
            country=self.country_gr,
            region=self.region_athens,
            city="Athens",
            zipcode="12345",
            phone="+306912345678",
            twitter="https://twitter.com/admin",
            linkedin="https://linkedin.com/in/admin",
            birth_date="1990-01-01",
        )
        self.admin_user.created_at = self.now - timedelta(days=60)
        self.admin_user.save()

        self.regular_user = UserAccountFactory(
            email="user@example.com",
            username="regular_user",
            first_name="Regular",
            last_name="User",
            is_staff=False,
            is_active=True,
            country=self.country_de,
            region=self.region_berlin,
            city="Berlin",
            zipcode="54321",
            phone=None,
            twitter="",
            linkedin="",
            birth_date=None,
        )
        self.regular_user.created_at = self.now - timedelta(hours=2)
        self.regular_user.save()

        self.inactive_user = UserAccountFactory(
            email="inactive@example.com",
            username="inactive_user",
            first_name="Inactive",
            last_name="User",
            is_staff=False,
            is_active=False,
            country=self.country_gr,
            region=self.region_athens,
            city="Thessaloniki",
            zipcode="67890",
            phone="+306987654321",
            github="https://github.com/inactive",
            website="https://inactive.com",
        )
        self.inactive_user.created_at = self.now - timedelta(hours=3)
        self.inactive_user.save()

        self.staff_user = UserAccountFactory(
            email="staff@example.com",
            username="staff_user",
            first_name="Staff",
            last_name="Member",
            is_staff=True,
            is_active=True,
            country=self.country_de,
            region=self.region_berlin,
            city="Munich",
            zipcode="98765",
            bio="I am a staff member",
            youtube="https://youtube.com/staff",
        )
        self.staff_user.created_at = self.now - timedelta(hours=4)
        self.staff_user.save()

    def test_basic_filters(self):
        url = reverse("user-account-list")

        response = self.client.get(url, {"id": self.admin_user.id})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.admin_user.id, result_ids)
        user_found = any(
            r["id"] == self.admin_user.id for r in response.data["results"]
        )
        self.assertTrue(user_found)

        response = self.client.get(url, {"is_active": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.admin_user.id, result_ids)
        self.assertIn(self.regular_user.id, result_ids)
        self.assertIn(self.staff_user.id, result_ids)

        response = self.client.get(url, {"is_staff": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_staff_users = [
            self.admin_user.id,
            self.staff_user.id,
            self.auth_user.id,
        ]
        for user_id in expected_staff_users:
            self.assertIn(user_id, result_ids)

    def test_email_filters(self):
        url = reverse("user-account-list")

        response = self.client.get(url, {"email_exact": "admin@example.com"})
        self.assertEqual(response.status_code, 200)
        result_emails = [r["email"] for r in response.data["results"]]
        self.assertIn("admin@example.com", result_emails)
        admin_found = any(
            r["email"] == "admin@example.com" for r in response.data["results"]
        )
        self.assertTrue(admin_found)

        response = self.client.get(url, {"email": "example.com"})
        self.assertEqual(response.status_code, 200)
        result_emails = [r["email"] for r in response.data["results"]]
        test_emails = [
            "admin@example.com",
            "user@example.com",
            "inactive@example.com",
            "staff@example.com",
            "auth@example.com",
        ]
        for email in test_emails:
            self.assertIn(email, result_emails)

        response = self.client.get(url, {"email": "admin"})
        self.assertEqual(response.status_code, 200)
        result_emails = [r["email"] for r in response.data["results"]]
        self.assertIn("admin@example.com", result_emails)

    def test_username_filters(self):
        url = reverse("user-account-list")

        response = self.client.get(url, {"username_exact": "admin_user"})
        self.assertEqual(response.status_code, 200)
        result_usernames = [r["username"] for r in response.data["results"]]
        self.assertIn("admin_user", result_usernames)
        admin_found = any(
            r["username"] == "admin_user" for r in response.data["results"]
        )
        self.assertTrue(admin_found)

        response = self.client.get(url, {"username": "user"})
        self.assertEqual(response.status_code, 200)
        result_usernames = [r["username"] for r in response.data["results"]]
        expected_usernames = [
            "admin_user",
            "regular_user",
            "inactive_user",
            "staff_user",
        ]
        for username in expected_usernames:
            self.assertIn(username, result_usernames)

    def test_name_filters(self):
        url = reverse("user-account-list")

        response = self.client.get(url, {"first_name": "Admin"})
        self.assertEqual(response.status_code, 200)
        result_first_names = [r["first_name"] for r in response.data["results"]]
        self.assertIn("Admin", result_first_names)
        admin_found = any(
            r["first_name"] == "Admin" for r in response.data["results"]
        )
        self.assertTrue(admin_found)

        response = self.client.get(url, {"last_name": "User"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_user_ids = [
            self.admin_user.id,
            self.regular_user.id,
            self.inactive_user.id,
        ]
        for user_id in expected_user_ids:
            self.assertIn(user_id, result_ids)

    def test_location_filters(self):
        url = reverse("user-account-list")

        response = self.client.get(url, {"country": "GR"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_gr_users = [self.admin_user.id, self.inactive_user.id]
        for user_id in expected_gr_users:
            self.assertIn(user_id, result_ids)

        response = self.client.get(url, {"region": "BER"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_ber_users = [self.regular_user.id, self.staff_user.id]
        for user_id in expected_ber_users:
            self.assertIn(user_id, result_ids)

        response = self.client.get(url, {"city": "Athens"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.admin_user.id, result_ids)

        response = self.client.get(url, {"zipcode": "123"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.admin_user.id, result_ids)

    def test_custom_filters(self):
        url = reverse("user-account-list")

        response = self.client.get(url, {"has_phone": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        expected_with_phone = [self.admin_user.id, self.inactive_user.id]
        for user_id in expected_with_phone:
            self.assertIn(user_id, result_ids)

        response = self.client.get(url, {"has_phone": "false"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.regular_user.id, result_ids)

        response = self.client.get(url, {"has_birth_date": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.admin_user.id, result_ids)

        response = self.client.get(url, {"has_social_links": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.admin_user.id, result_ids)
        self.assertIn(self.inactive_user.id, result_ids)
        self.assertIn(self.staff_user.id, result_ids)

    def test_timestamp_filters(self):
        url = reverse("user-account-list")

        created_after_date = self.now - timedelta(days=30)
        response = self.client.get(
            url,
            {"created_after": created_after_date.isoformat()},
        )
        self.assertEqual(response.status_code, 200)

        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.regular_user.id, result_ids)
        self.assertIn(self.inactive_user.id, result_ids)
        self.assertIn(self.staff_user.id, result_ids)

        created_before_date = self.now - timedelta(days=30)
        response = self.client.get(
            url,
            {"created_before": created_before_date.isoformat()},
        )
        self.assertEqual(response.status_code, 200)

        result_ids = [r["id"] for r in response.data["results"]]
        self.assertGreaterEqual(len(result_ids), 0)

    def test_uuid_filter(self):
        url = reverse("user-account-list")

        response = self.client.get(url, {"uuid": str(self.regular_user.uuid)})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.regular_user.id, result_ids)
        user_found = any(
            r["id"] == self.regular_user.id for r in response.data["results"]
        )
        self.assertTrue(user_found)

    def test_camel_case_filters(self):
        url = reverse("user-account-list")

        created_after_date = self.now - timedelta(days=30)
        response = self.client.get(
            url,
            {
                "createdAfter": created_after_date.isoformat(),
                "isActive": "true",
            },
        )
        self.assertEqual(response.status_code, 200)

        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.regular_user.id, result_ids)
        self.assertIn(self.staff_user.id, result_ids)
        self.assertIn(self.auth_user.id, result_ids)
        recent_active_users = [
            self.regular_user.id,
            self.staff_user.id,
            self.auth_user.id,
        ]
        for user_id in recent_active_users:
            self.assertIn(user_id, result_ids)

        response = self.client.get(
            url,
            {
                "isStaff": "true",
                "hasPhone": "true",
            },
        )
        self.assertEqual(response.status_code, 200)

        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.admin_user.id, result_ids)

    def test_existing_filters_still_work(self):
        url = reverse("user-account-list")

        response = self.client.get(
            url,
            {"is_staff": "true", "has_phone": "true", "country": "GR"},
        )
        self.assertEqual(response.status_code, 200)

        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.admin_user.id, result_ids)
        admin_result = next(
            (
                r
                for r in response.data["results"]
                if r["id"] == self.admin_user.id
            ),
            None,
        )
        self.assertIsNotNone(admin_result)

    def test_bulk_filters(self):
        url = reverse("user-account-list")

        response = self.client.get(
            url,
            {
                "is_active": "true",
                "country": "DE",
                "city": "Berlin",
                "has_phone": "false",
            },
        )
        self.assertEqual(response.status_code, 200)

        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.regular_user.id, result_ids)
        regular_result = next(
            (
                r
                for r in response.data["results"]
                if r["id"] == self.regular_user.id
            ),
            None,
        )
        self.assertIsNotNone(regular_result)

    def test_complex_filter_combinations(self):
        url = reverse("user-account-list")

        created_after_date = self.now - timedelta(days=90)

        response = self.client.get(
            url,
            {
                "createdAfter": created_after_date.isoformat(),
                "isActive": "true",
                "hasSocialLinks": "true",
                "ordering": "-createdAt",
            },
        )

        self.assertEqual(response.status_code, 200)

        result_ids = [r["id"] for r in response.data["results"]]
        self.assertIn(self.admin_user.id, result_ids)
        self.assertIn(self.staff_user.id, result_ids)
        expected_with_social = [
            self.admin_user.id,
            self.inactive_user.id,
            self.staff_user.id,
        ]
        for user_id in expected_with_social:
            self.assertIn(user_id, result_ids)

        results = response.data["results"]
        self.assertGreater(len(results), 0)

    def test_filter_with_ordering(self):
        url = reverse("user-account-list")

        response = self.client.get(
            url, {"isActive": "true", "ordering": "-email"}
        )
        self.assertEqual(response.status_code, 200)

        results = response.data["results"]
        result_ids = [r["id"] for r in results]
        active_test_users = [
            self.admin_user.id,
            self.regular_user.id,
            self.staff_user.id,
            self.auth_user.id,
        ]
        for user_id in active_test_users:
            self.assertIn(user_id, result_ids)

        emails = [r["email"] for r in results]
        self.assertEqual(emails, sorted(emails, reverse=True))

    def test_country_name_filter(self):
        url = reverse("user-account-list")

        response = self.client.get(url, {"country_name": "Greece"})
        self.assertEqual(response.status_code, 200)

        result_ids = [r["id"] for r in response.data["results"]]
        expected_greece_users = [self.admin_user.id, self.inactive_user.id]
        for user_id in expected_greece_users:
            self.assertIn(user_id, result_ids)

    def tearDown(self):
        UserAccount.objects.all().delete()



from functools import cached_property
from typing import Any

from django.conf import settings
from django.contrib.postgres.indexes import BTreeIndex, GinIndex
from django.core.exceptions import ValidationError
from django.core.validators import validate_email
from django.db import models
from django.db.models import F, Sum
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django_stubs_ext.db.models import TypedModelMeta
from djmoney.models.fields import MoneyField
from djmoney.money import Money
from phonenumber_field.modelfields import PhoneNumberField

from core.enum import FloorChoicesEnum, LocationChoicesEnum
from core.models import SoftDeleteModel, TimeStampMixinModel, UUIDModel
from order.enum.document_type import OrderDocumentTypeEnum
from order.enum.status import OrderStatus, PaymentStatus
from order.managers.order import OrderManager


class Order(SoftDeleteModel, TimeStampMixinModel, UUIDModel):
    id = models.BigAutoField(primary_key=True)
    user = models.ForeignKey(
        "user.UserAccount",
        related_name="orders",
        on_delete=models.CASCADE,
        null=True,
        blank=True,
    )
    pay_way = models.ForeignKey(
        "pay_way.PayWay",
        related_name="orders",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
    )
    country = models.ForeignKey(
        "country.Country",
        related_name="orders",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
    )
    region = models.ForeignKey(
        "region.Region",
        related_name="orders",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
    )
    floor = models.CharField(
        _("Floor"),
        max_length=50,
        choices=FloorChoicesEnum,
        blank=True,
        default="",
    )
    location_type = models.CharField(
        _("Location Type"),
        max_length=100,
        choices=LocationChoicesEnum,
        blank=True,
        default="",
    )
    email = models.EmailField(_("Email"), max_length=255)
    first_name = models.CharField(_("First Name"), max_length=255)
    last_name = models.CharField(_("Last Name"), max_length=255)
    street = models.CharField(_("Street"), max_length=255)
    street_number = models.CharField(_("Street Number"), max_length=255)
    city = models.CharField(_("City"), max_length=255)
    zipcode = models.CharField(_("Zipcode"), max_length=255)
    place = models.CharField(_("Place"), max_length=255, blank=True, default="")
    phone = PhoneNumberField(_("Phone Number"))
    mobile_phone = PhoneNumberField(
        _("Mobile Phone Number"), null=True, blank=True, default=None
    )
    customer_notes = models.TextField(
        _("Customer Notes"), blank=True, default=""
    )
    status = models.CharField(
        _("Status"),
        max_length=20,
        choices=OrderStatus,
        default=OrderStatus.PENDING,
    )
    shipping_price = MoneyField(
        _("Shipping Price"), max_digits=11, decimal_places=2, default=0
    )
    document_type = models.CharField(
        _("Document Type"),
        max_length=100,
        choices=OrderDocumentTypeEnum,
        default=OrderDocumentTypeEnum.RECEIPT,
    )
    paid_amount = MoneyField(
        _("Paid Amount"),
        max_digits=11,
        decimal_places=2,
        null=True,
        default=0,
    )
    status_updated_at = models.DateTimeField(
        _("Status Updated At"), auto_now=False, null=True, blank=True
    )
    payment_id = models.CharField(
        _("Payment ID"), max_length=255, blank=True, default=""
    )
    payment_status = models.CharField(
        _("Payment Status"),
        max_length=50,
        blank=True,
        choices=PaymentStatus,
        default=PaymentStatus.PENDING,
    )
    payment_method = models.CharField(
        _("Payment Method"), max_length=50, blank=True, default=""
    )
    tracking_number = models.CharField(
        _("Tracking Number"), max_length=255, blank=True, default=""
    )
    shipping_carrier = models.CharField(
        _("Shipping Carrier"), max_length=255, blank=True, default=""
    )

    objects: OrderManager = OrderManager()

    class Meta(TypedModelMeta):
        verbose_name = _("Order")
        verbose_name_plural = _("Orders")
        ordering = ["-created_at"]
        indexes = [
            *TimeStampMixinModel.Meta.indexes,
            BTreeIndex(fields=["status"], name="order_status_ix"),
            BTreeIndex(fields=["document_type"], name="order_doc_type_ix"),
            BTreeIndex(
                fields=["status_updated_at"], name="order_status_upd_ix"
            ),
            BTreeIndex(
                fields=["payment_status"], name="order_payment_status_ix"
            ),
            BTreeIndex(fields=["user"], name="order_user_ix"),
            BTreeIndex(fields=["pay_way"], name="order_pay_way_ix"),
            BTreeIndex(fields=["country"], name="order_country_ix"),
            BTreeIndex(fields=["region"], name="order_region_ix"),
            BTreeIndex(fields=["user", "status"], name="order_user_status_ix"),
            BTreeIndex(
                fields=["status", "payment_status"],
                name="order_status_payment_ix",
            ),
            BTreeIndex(
                fields=["tracking_number"], name="order_tracking_num_ix"
            ),
            BTreeIndex(fields=["payment_id"], name="order_payment_id_ix"),
            GinIndex(
                name="order_name_search_ix",
                fields=["first_name", "last_name"],
                opclasses=["gin_trgm_ops"] * 2,
            ),
            GinIndex(
                name="order_email_search_ix",
                fields=["email"],
                opclasses=["gin_trgm_ops"],
            ),
            GinIndex(
                name="order_address_street_ix",
                fields=["street"],
                opclasses=["gin_trgm_ops"],
            ),
            GinIndex(
                name="order_address_location_ix",
                fields=["city", "place"],
                opclasses=["gin_trgm_ops"] * 2,
            ),
        ]

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self._original_status = self.status

    def __str__(self) -> str:
        return f"Order {self.id} - {self.first_name} {self.last_name}"

    def save(self, *args: Any, **kwargs: Any) -> None:
        if (
            self.pk
            and hasattr(self, "_original_status")
            and self.status != self._original_status
        ):
            self.status_updated_at = timezone.now()

        if (
            not self.email
            and self.user_id is not None
            and hasattr(self, "user")
            and self.user is not None
        ):
            self.email = self.user.email

        super().save(*args, **kwargs)
        self._original_status = self.status

    def clean(self) -> None:
        errors: dict[str, list[str]] = {}

        if self.email:
            try:
                validate_email(self.email)
            except ValidationError:
                errors["email"] = [_("Enter a valid email address.")]

        if not self.zipcode:
            errors["zipcode"] = [_("Zipcode is required.")]

        required_address_fields = ["street", "street_number", "city"]
        missing_fields = [
            field
            for field in required_address_fields
            if not getattr(self, field)
        ]

        if missing_fields:
            errors["address"] = [
                _("Street, street number, and city are required.")
            ]

        if errors:
            raise ValidationError(errors)

    @property
    def total_price_items(self) -> Money:
        items_total = self.items.aggregate(
            total=Sum(F("price") * F("quantity"))
        ).get("total")

        if not items_total:
            default_currency = getattr(settings, "DEFAULT_CURRENCY", "EUR")
            if self.shipping_price:
                return Money(0, self.shipping_price.currency)
            return Money(0, default_currency)

        first_item = self.items.first()
        currency = (
            first_item.price.currency
            if first_item
            else getattr(settings, "DEFAULT_CURRENCY", "EUR")
        )

        return Money(amount=items_total, currency=currency)

    @property
    def total_price_extra(self) -> Money:
        return self.shipping_price

    @property
    def full_address(self) -> str:
        return f"{self.street} {self.street_number}, {self.zipcode} {self.city}"

    @property
    def customer_full_name(self) -> str:
        return f"{self.first_name} {self.last_name}"

    @property
    def is_paid(self) -> bool:
        return bool(
            (
                self.payment_status
                and self.payment_status == PaymentStatus.COMPLETED
            )
            or (self.paid_amount and self.paid_amount.amount > 0)
        )

    @property
    def can_be_canceled(self) -> bool:
        cancellable_statuses = [
            OrderStatus.PENDING,
            OrderStatus.PROCESSING,
        ]
        return self.status in cancellable_statuses

    @property
    def is_completed(self) -> bool:
        return self.status == OrderStatus.COMPLETED

    @property
    def is_canceled(self) -> bool:
        return self.status == OrderStatus.CANCELED

    @cached_property
    def total_price(self) -> Money:
        items_total = self.total_price_items
        extras_total = self.total_price_extra

        if items_total.currency != extras_total.currency:
            raise ValueError(
                f"Items and extras have different currencies: {items_total.currency} and {extras_total.currency}"
            )

        return Money(
            items_total.amount + extras_total.amount, items_total.currency
        )

    def calculate_order_total_amount(self) -> Money:
        return self.total_price

    def mark_as_paid(
        self,
        payment_id: str | None = None,
        payment_method: str | None = None,
    ) -> None:
        self.payment_status = PaymentStatus.COMPLETED
        if payment_id:
            self.payment_id = payment_id
        if payment_method:
            self.payment_method = payment_method

        if not self.paid_amount or self.paid_amount.amount == 0:
            self.paid_amount = self.calculate_order_total_amount()

        self.save(
            update_fields=[
                "payment_status",
                "payment_id",
                "payment_method",
                "paid_amount",
            ]
        )

    def add_tracking_info(
        self, tracking_number: str, shipping_carrier: str
    ) -> None:
        self.tracking_number = tracking_number
        self.shipping_carrier = shipping_carrier
        self.save(update_fields=["tracking_number", "shipping_carrier"])



# Generated by Django 4.2.9 on 2024-02-10 14:39
import django.db.models.deletion
import parler.fields
from django.conf import settings
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("notification", "0001_initial"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name="notificationuser",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="notification",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AddField(
            model_name="notificationtranslation",
            name="master",
            field=parler.fields.TranslationsForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="translations",
                to="notification.notification",
            ),
        ),
        migrations.AddConstraint(
            model_name="notificationuser",
            constraint=models.UniqueConstraint(
                fields=("user", "notification"),
                name="unique_notification_user",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="notificationtranslation",
            unique_together={("language_code", "master")},
        ),
    ]



from io import StringIO
from unittest.mock import MagicMock, patch

from django.core.management import call_command
from django.test import TestCase

from core.management.commands.clear_meilisearch import Command


class TestClearMeiliSearchCommand(TestCase):
    def setUp(self):
        self.command = Command()

    def test_help_message(self):
        assert (
            self.command.help
            == "Clears all MeiliSearch indexes and data (equivalent to clearing the MeiliSearch database)"
        )

    @patch("core.management.commands.clear_meilisearch._client")
    @patch("builtins.input", return_value="yes")
    def test_clear_meilisearch_success(self, mock_input, mock_client):
        mock_index1 = MagicMock()
        mock_index1.uid = "test_index_1"
        mock_index2 = MagicMock()
        mock_index2.uid = "test_index_2"

        mock_client.get_indexes.return_value = [mock_index1, mock_index2]

        mock_task = MagicMock()
        mock_task.task_uid = "task_123"
        mock_client.client.delete_index.return_value = mock_task

        mock_finished_task = MagicMock()
        mock_finished_task.status = "succeeded"
        mock_client.wait_for_task.return_value = mock_finished_task

        output = StringIO()
        call_command("clear_meilisearch", stdout=output)

        output_content = output.getvalue()
        assert "Found 2 indexes to delete" in output_content
        assert "Deleted index: test_index_1" in output_content
        assert "Deleted index: test_index_2" in output_content
        assert "Successfully deleted 2 indexes" in output_content

    @patch("core.management.commands.clear_meilisearch._client")
    def test_clear_meilisearch_force_flag(self, mock_client):
        mock_client.get_indexes.return_value = []

        output = StringIO()
        call_command("clear_meilisearch", "--force", stdout=output)

        output_content = output.getvalue()
        assert "No indexes found in MeiliSearch" in output_content

    @patch("core.management.commands.clear_meilisearch._client")
    @patch("builtins.input", return_value="no")
    def test_clear_meilisearch_cancelled(self, mock_input, mock_client):
        output = StringIO()
        call_command("clear_meilisearch", stdout=output)

        output_content = output.getvalue()
        assert "Operation cancelled" in output_content

    @patch("core.management.commands.clear_meilisearch._client")
    @patch("builtins.input", return_value="yes")
    def test_clear_meilisearch_with_failed_deletion(
        self, mock_input, mock_client
    ):
        mock_index = MagicMock()
        mock_index.uid = "test_index"
        mock_client.get_indexes.return_value = [mock_index]

        mock_task = MagicMock()
        mock_task.task_uid = "task_123"
        mock_client.client.delete_index.return_value = mock_task

        mock_finished_task = MagicMock()
        mock_finished_task.status = "failed"
        mock_finished_task.error = "Deletion failed"
        mock_client.wait_for_task.return_value = mock_finished_task

        output = StringIO()
        stderr = StringIO()
        call_command("clear_meilisearch", stdout=output, stderr=stderr)

        stderr_content = stderr.getvalue()
        assert (
            "Failed to delete index 'test_index': Deletion failed"
            in stderr_content
        )

    @patch("core.management.commands.clear_meilisearch._client")
    @patch("builtins.input", return_value="yes")
    def test_clear_meilisearch_with_exception(self, mock_input, mock_client):
        mock_client.get_indexes.side_effect = Exception("Connection error")

        output = StringIO()
        stderr = StringIO()
        call_command("clear_meilisearch", stdout=output, stderr=stderr)

        stderr_content = stderr.getvalue()
        assert "Error clearing MeiliSearch: Connection error" in stderr_content

    @patch("core.management.commands.clear_meilisearch._client")
    @patch("core.management.commands.clear_meilisearch.apps")
    @patch("builtins.input", return_value="yes")
    def test_clear_meilisearch_with_recreate(
        self, mock_input, mock_apps, mock_client
    ):
        mock_index = MagicMock()
        mock_index.uid = "test_index"
        mock_client.get_indexes.return_value = [mock_index]

        mock_task = MagicMock()
        mock_task.task_uid = "task_123"
        mock_client.client.delete_index.return_value = mock_task

        mock_finished_task = MagicMock()
        mock_finished_task.status = "succeeded"
        mock_client.wait_for_task.return_value = mock_finished_task

        from meili.models import IndexMixin

        mock_model = MagicMock()
        mock_model.__mro__ = (IndexMixin, object)
        mock_model.__name__ = "TestModel"
        mock_model._meilisearch = {
            "index_name": "test_index",
            "primary_key": "pk",
        }

        mock_app_config = MagicMock()
        mock_app_config.get_models.return_value = [mock_model]
        mock_apps.get_app_configs.return_value = [mock_app_config]

        output = StringIO()
        call_command("clear_meilisearch", "--recreate", stdout=output)

        output_content = output.getvalue()
        assert "Recreating indexes..." in output_content
        assert "Recreated index: test_index" in output_content



from dataclasses import dataclass
from typing import TypedDict, cast


class MeiliSettings(TypedDict):
    HTTPS: bool
    HOST: str
    MASTER_KEY: str
    PORT: int
    TIMEOUT: int | None
    CLIENT_AGENTS: tuple[str] | None
    DEBUG: bool | None
    SYNC: bool | None
    OFFLINE: bool | None
    DEFAULT_BATCH_SIZE: int


@dataclass(frozen=True, slots=True)
class _MeiliSettings:
    https: bool
    host: str
    master_key: str
    port: int
    timeout: int | None
    client_agents: tuple[str] | None
    debug: bool
    sync: bool
    offline: bool
    batch_size: int

    @classmethod
    def from_settings(cls) -> "_MeiliSettings":
        from django.conf import settings  # noqa: PLC0415

        meili_settings = cast("MeiliSettings", settings.MEILISEARCH)

        master_key = meili_settings.get("MASTER_KEY")
        if not master_key:
            raise ValueError("MEILISEARCH['MASTER_KEY'] is required")

        debug = meili_settings.get("DEBUG")
        sync = meili_settings.get("SYNC")
        offline = meili_settings.get("OFFLINE")

        return cls(
            https=meili_settings.get("HTTPS", False),
            host=meili_settings.get("HOST", "localhost"),
            master_key=master_key,
            port=meili_settings.get("PORT", 7700),
            timeout=meili_settings.get("TIMEOUT", None),
            client_agents=meili_settings.get("CLIENT_AGENTS", None),
            debug=settings.DEBUG if debug is None else debug,
            sync=False if sync is None else sync,
            offline=False if offline is None else offline,
            batch_size=meili_settings.get("DEFAULT_BATCH_SIZE", 1000),
        )



from django.test import TestCase, override_settings


class DebugToolbarURLsTestCase(TestCase):
    @override_settings(DEBUG=False, ENABLE_DEBUG_TOOLBAR=True)
    def test_no_debug_toolbar_url_when_debug_false(self):
        response = self.client.get("/__debug__/", follow=True)
        self.assertEqual(response.status_code, 404)



from __future__ import annotations

from django.conf import settings
from django.contrib.auth import get_user_model
from drf_spectacular.utils import extend_schema_field
from phonenumber_field.serializerfields import PhoneNumberField
from rest_framework import serializers
from django.utils.translation import gettext_lazy as _

User = get_user_model()


class UserSerializer(serializers.ModelSerializer[User]):
    def validate_username(self, username: str) -> str:
        if "allauth.account" not in settings.INSTALLED_APPS:
            return username

        from allauth.account.adapter import get_adapter  # noqa: PLC0415

        username = get_adapter().clean_username(username)
        return username

    class Meta:
        extra_fields = []
        if hasattr(User, "USERNAME_FIELD"):
            extra_fields.append(User.USERNAME_FIELD)
        if hasattr(User, "EMAIL_FIELD"):
            extra_fields.append(User.EMAIL_FIELD)
        if hasattr(User, "first_name"):
            extra_fields.append("first_name")
        if hasattr(User, "last_name"):
            extra_fields.append("last_name")
        model = User
        fields = ("pk", *extra_fields)
        read_only_fields = ("email",)


class UserWriteSerializer(UserSerializer):
    phone = PhoneNumberField(required=False, allow_blank=True, allow_null=True)

    class Meta(UserSerializer.Meta):
        fields = (
            *UserSerializer.Meta.fields,
            "email",
            "username",
            "image",
            "first_name",
            "last_name",
            "phone",
            "city",
            "zipcode",
            "address",
            "place",
            "country",
            "region",
            "birth_date",
            "twitter",
            "linkedin",
            "facebook",
            "instagram",
            "website",
            "youtube",
            "github",
            "bio",
        )
        read_only_fields = (
            "created_at",
            "updated_at",
            "uuid",
        )

    def validate_email(self, email):
        if self.instance and self.instance.email != email:
            if User.objects.filter(email=email).exists():
                raise serializers.ValidationError(
                    _("A user with this email already exists.")
                )
        return email

    def validate_username(self, username):
        username = super().validate_username(username)
        if self.instance and self.instance.username != username:
            if User.objects.filter(username=username).exists():
                raise serializers.ValidationError(
                    _("A user with this username already exists.")
                )
        return username


class UserDetailsSerializer(UserSerializer):
    phone = PhoneNumberField(required=False, allow_blank=True, allow_null=True)
    twitter = serializers.SerializerMethodField()
    linkedin = serializers.SerializerMethodField()
    facebook = serializers.SerializerMethodField()
    instagram = serializers.SerializerMethodField()
    website = serializers.SerializerMethodField()
    youtube = serializers.SerializerMethodField()
    github = serializers.SerializerMethodField()

    @extend_schema_field(
        {
            "type": "string",
            "nullable": True,
            "maxLength": 200,
            "description": _("URL link or empty string"),
        }
    )
    def get_twitter(self, obj) -> str | None:
        return obj.twitter

    @extend_schema_field(
        {
            "type": "string",
            "nullable": True,
            "maxLength": 200,
            "description": _("URL link or empty string"),
        }
    )
    def get_linkedin(self, obj) -> str | None:
        return obj.linkedin

    @extend_schema_field(
        {
            "type": "string",
            "nullable": True,
            "maxLength": 200,
            "description": _("URL link or empty string"),
        }
    )
    def get_facebook(self, obj) -> str | None:
        return obj.facebook

    @extend_schema_field(
        {
            "type": "string",
            "nullable": True,
            "maxLength": 200,
            "description": _("URL link or empty string"),
        }
    )
    def get_instagram(self, obj) -> str | None:
        return obj.instagram

    @extend_schema_field(
        {
            "type": "string",
            "nullable": True,
            "maxLength": 200,
            "description": _("URL link or empty string"),
        }
    )
    def get_website(self, obj) -> str | None:
        return obj.website

    @extend_schema_field(
        {
            "type": "string",
            "nullable": True,
            "maxLength": 200,
            "description": _("URL link or empty string"),
        }
    )
    def get_youtube(self, obj) -> str | None:
        return obj.youtube

    @extend_schema_field(
        {
            "type": "string",
            "nullable": True,
            "maxLength": 200,
            "description": _("URL link or empty string"),
        }
    )
    def get_github(self, obj) -> str | None:
        return obj.github

    class Meta(UserSerializer.Meta):
        fields = (
            *UserSerializer.Meta.fields,
            "id",
            "email",
            "username",
            "first_name",
            "last_name",
            "phone",
            "city",
            "zipcode",
            "address",
            "place",
            "country",
            "region",
            "birth_date",
            "twitter",
            "linkedin",
            "facebook",
            "instagram",
            "website",
            "youtube",
            "github",
            "bio",
            "is_active",
            "is_staff",
            "is_superuser",
            "created_at",
            "updated_at",
            "uuid",
            "main_image_path",
        )
        read_only_fields = (
            "is_active",
            "is_staff",
            "is_superuser",
            "created_at",
            "updated_at",
            "uuid",
            "full_name",
            "main_image_path",
        )


class UsernameUpdateSerializer(serializers.Serializer):
    username = serializers.CharField(
        max_length=150,
        help_text=_("New username"),
    )


class UsernameUpdateResponseSerializer(serializers.Serializer):
    detail = serializers.CharField(
        help_text=_("Success message for username update")
    )


class UserSubscriptionSummaryResponseSerializer(serializers.Serializer):
    total_subscriptions = serializers.IntegerField()
    active_subscriptions = serializers.IntegerField()
    categories = serializers.ListField(child=serializers.CharField())



from __future__ import annotations

import os

from django.contrib.postgres.indexes import BTreeIndex
from django.db import models
from django.utils.translation import gettext_lazy as _
from django_stubs_ext.db.models import TypedModelMeta
from parler.models import TranslatableModel, TranslatedFields

from core.fields.image import ImageAndSvgField
from core.models import SortableModel, TimeStampMixinModel, UUIDModel
from product.enum.category import CategoryImageTypeEnum
from product.managers.category_image import CategoryImageManager


class ProductCategoryImage(
    TranslatableModel, TimeStampMixinModel, SortableModel, UUIDModel
):
    id = models.BigAutoField(primary_key=True)
    category = models.ForeignKey(
        "product.ProductCategory",
        related_name="images",
        on_delete=models.CASCADE,
    )
    image = ImageAndSvgField(_("Image"), upload_to="uploads/categories/")
    image_type = models.CharField(
        _("Image Type"),
        max_length=20,
        choices=CategoryImageTypeEnum.choices,
        default=CategoryImageTypeEnum.MAIN,
    )
    active = models.BooleanField(_("Active"), default=True)
    translations = TranslatedFields(
        title=models.CharField(
            _("Title"), max_length=255, blank=True, null=True
        ),
        alt_text=models.CharField(
            _("Alt Text"), max_length=255, blank=True, null=True
        ),
    )

    objects: CategoryImageManager = CategoryImageManager()

    class Meta(TypedModelMeta):
        verbose_name = _("Product Category Image")
        verbose_name_plural = _("Product Category Images")
        ordering = ["sort_order"]
        unique_together = [["category", "image_type"]]
        indexes = [
            BTreeIndex(fields=["created_at"], name="cat_img_created_ix"),
            BTreeIndex(fields=["updated_at"], name="cat_img_updated_ix"),
            BTreeIndex(fields=["sort_order"], name="cat_img_sort_ix"),
            BTreeIndex(
                fields=["category", "image_type"],
                name="cat_img_category_type_ix",
            ),
            BTreeIndex(
                fields=["category", "active"],
                name="cat_img_category_active_ix",
            ),
            BTreeIndex(
                fields=["image_type", "active"],
                name="cat_img_type_active_ix",
            ),
        ]

    def __str__(self):
        category_name = self.category.safe_translation_getter(
            "name", any_language=True
        )
        return f"{category_name} - {self.get_image_type_display()}"

    def get_ordering_queryset(self):
        return self.category.images.filter(image_type=self.image_type)

    def clean(self):
        super().clean()

    @property
    def image_path(self) -> str:
        if self.image and hasattr(self.image, "name"):
            return (
                f"media/uploads/categories/{os.path.basename(self.image.name)}"
            )
        return ""

    @property
    def image_url(self) -> str:
        if self.image and hasattr(self.image, "url"):
            return self.image.url
        return ""

    @classmethod
    def get_main_image(cls, category):
        return cls.objects.get_main_image(category)

    @classmethod
    def get_banner_image(cls, category):
        return cls.objects.get_banner_image(category)

    @classmethod
    def get_icon_image(cls, category):
        return cls.objects.get_icon_image(category)

    @classmethod
    def get_image_by_type(cls, category, image_type: CategoryImageTypeEnum):
        return cls.objects.get_image_by_type(category, image_type)



from django.urls import reverse
from rest_framework.test import APITestCase

from cart.factories.cart import CartFactory
from user.factories.account import UserAccountFactory


class CartFilterTest(APITestCase):
    def setUp(self):
        self.user = UserAccountFactory(is_staff=True, is_superuser=True)
        self.client.force_authenticate(user=self.user)

        self.cart1 = CartFactory()
        self.cart2 = CartFactory()

    def test_activity_filters(self):
        url = reverse("cart-list")

        response = self.client.get(
            url, {"last_activity_after": "2024-01-01T00:00:00Z"}
        )
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

        response = self.client.get(
            url, {"last_activity_before": "2025-12-31T23:59:59Z"}
        )
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

    def test_user_filters(self):
        url = reverse("cart-list")

        response = self.client.get(url, {"user": 1})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

        response = self.client.get(url, {"has_user": "true"})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

    def test_session_filters(self):
        url = reverse("cart-list")

        response = self.client.get(url, {"session_key": "test-session"})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

        response = self.client.get(url, {"has_session": "true"})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

    def test_cart_type_filters(self):
        url = reverse("cart-list")

        response = self.client.get(url, {"cart_type": "user"})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

        response = self.client.get(url, {"cart_type": "guest"})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

    def test_item_filters(self):
        url = reverse("cart-list")

        response = self.client.get(url, {"has_items": "true"})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

        response = self.client.get(url, {"min_items": 1})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

        response = self.client.get(url, {"max_items": 10})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

    def test_value_filters(self):
        url = reverse("cart-list")

        response = self.client.get(url, {"min_total_price": "10.00"})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

        response = self.client.get(url, {"max_total_price": "1000.00"})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

    def test_timestamp_filters(self):
        url = reverse("cart-list")

        response = self.client.get(
            url, {"created_after": "2024-01-01T00:00:00Z"}
        )
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

        response = self.client.get(
            url, {"created_before": "2025-12-31T23:59:59Z"}
        )
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

    def test_uuid_filter(self):
        url = reverse("cart-list")

        response = self.client.get(url, {"uuid": str(self.cart1.uuid)})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

    def test_camel_case_filters(self):
        url = reverse("cart-list")

        response = self.client.get(
            url, {"hasUser": "true", "hasItems": "true", "cartType": "user"}
        )
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

    def test_complex_filter_combinations(self):
        url = reverse("cart-list")

        response = self.client.get(
            url,
            {
                "has_user": "true",
                "has_items": "true",
                "min_total_price": "10.00",
            },
        )
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

    def test_filter_with_ordering(self):
        url = reverse("cart-list")

        response = self.client.get(
            url, {"has_items": "true", "ordering": "-created_at"}
        )
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)



# Generated by Django 5.0.6 on 2024-07-07 14:34
import django.db.models.deletion
from django.conf import settings
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("blog", "0016_alter_blogcategory_slug_alter_blogpost_image_and_more"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AlterField(
            model_name="blogcomment",
            name="likes",
            field=models.ManyToManyField(
                blank=True,
                related_name="liked_blog_comments",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="blogcomment",
            name="post",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="comments",
                to="blog.blogpost",
            ),
        ),
        migrations.AlterField(
            model_name="blogcomment",
            name="user",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="blog_comments",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="blogpost",
            name="author",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="blog_posts",
                to="blog.blogauthor",
            ),
        ),
        migrations.AlterField(
            model_name="blogpost",
            name="category",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="blog_posts",
                to="blog.blogcategory",
            ),
        ),
        migrations.AlterField(
            model_name="blogpost",
            name="tags",
            field=models.ManyToManyField(
                blank=True, related_name="blog_posts", to="blog.blogtag"
            ),
        ),
    ]



from django.dispatch import Signal

order_created = Signal()
order_status_changed = Signal()
order_paid = Signal()
order_canceled = Signal()
order_shipped = Signal()
order_delivered = Signal()
order_completed = Signal()
order_refunded = Signal()
order_returned = Signal()



from __future__ import annotations

from django.db import models
from django.utils.translation import gettext_lazy as _
from drf_spectacular.utils import extend_schema, extend_schema_view
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.response import Response

from core.api.permissions import IsOwnerOrAdmin
from core.api.serializers import ErrorResponseSerializer
from core.api.views import BaseModelViewSet
from core.utils.serializers import (
    create_schema_view_config,
    RequestSerializersConfig,
    ResponseSerializersConfig,
)
from product.enum.review import ReviewStatus
from product.filters.review import ProductReviewFilter
from product.models.review import ProductReview
from product.serializers.product import ProductSerializer
from product.serializers.review import (
    ProductReviewDetailSerializer,
    ProductReviewSerializer,
    ProductReviewWriteSerializer,
)

req_serializers: RequestSerializersConfig = {
    "create": ProductReviewWriteSerializer,
    "update": ProductReviewWriteSerializer,
    "partial_update": ProductReviewWriteSerializer,
}

res_serializers: ResponseSerializersConfig = {
    "create": ProductReviewDetailSerializer,
    "list": ProductReviewSerializer,
    "retrieve": ProductReviewDetailSerializer,
    "update": ProductReviewDetailSerializer,
    "partial_update": ProductReviewDetailSerializer,
    "user_product_review": ProductReviewDetailSerializer,
    "product": ProductSerializer,
}


@extend_schema_view(
    **create_schema_view_config(
        model_class=ProductReview,
        display_config={
            "tag": "Product Reviews",
        },
        request_serializers=req_serializers,
        response_serializers=res_serializers,
        error_serializer=ErrorResponseSerializer,
    )
)
class ProductReviewViewSet(BaseModelViewSet):
    filterset_class = ProductReviewFilter
    ordering_fields = [
        "id",
        "user_id",
        "product_id",
        "rate",
        "status",
        "is_published",
        "created_at",
        "updated_at",
        "published_at",
    ]
    ordering = ["-created_at"]
    search_fields = [
        "product__translations__name",
        "user__email",
        "user__first_name",
        "user__last_name",
        "translations__comment",
    ]
    response_serializers = res_serializers
    request_serializers = req_serializers

    def get_queryset(self):
        queryset = ProductReview.objects.with_product_details()

        if self.request.user.is_superuser:
            return queryset

        if self.request.user.is_authenticated:
            return queryset.filter(
                models.Q(status=ReviewStatus.TRUE)
                | models.Q(user=self.request.user)
            )
        else:
            return queryset.filter(status=ReviewStatus.TRUE)

    def get_permissions(self):
        if self.action in [
            "create",
            "update",
            "partial_update",
            "destroy",
            "user_product_review",
        ]:
            self.permission_classes = [IsOwnerOrAdmin]
        return super().get_permissions()

    def perform_create(self, serializer):
        serializer.save(user=self.request.user)

    @extend_schema(
        operation_id="getUserProductReview",
        summary=_("Get user's review for a product"),
        description=_(
            "Get the current user's review for a specific product. Requires authentication."
        ),
        tags=["Product Reviews"],
        responses={
            200: ProductReviewDetailSerializer,
            400: ErrorResponseSerializer,
            401: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    )
    @action(detail=True, methods=["GET"])
    def user_product_review(self, request, *args, **kwargs):
        if not request.user.is_authenticated:
            return Response(
                {"detail": _("User is not authenticated")},
                status=status.HTTP_401_UNAUTHORIZED,
            )

        user_id = request.user.id

        try:
            review = ProductReview.objects.get(
                user_id=user_id, product_id=kwargs["pk"]
            )
            response_serializer_class = self.get_response_serializer()
            response_serializer = response_serializer_class(
                review, context=self.get_serializer_context()
            )
            return Response(response_serializer.data, status=status.HTTP_200_OK)

        except ProductReview.DoesNotExist:
            return Response(
                {"detail": _("Review does not exist")},
                status=status.HTTP_404_NOT_FOUND,
            )

    @extend_schema(
        operation_id="getProductReviewProduct",
        summary=_("Get reviewed product details"),
        description=_(
            "Get detailed information about the product this review is for."
        ),
        tags=["Product Reviews"],
        responses={
            200: ProductSerializer,
            404: ErrorResponseSerializer,
        },
    )
    @action(detail=True, methods=["GET"])
    def product(self, request, *args, **kwargs):
        review = self.get_object()

        self.ordering_fields = []
        self.ordering = []
        self.search_fields = []

        product = review.product
        response_serializer_class = self.get_response_serializer()
        response_serializer = response_serializer_class(
            product, context=self.get_serializer_context()
        )
        return Response(response_serializer.data, status=status.HTTP_200_OK)



# Generated by Django 5.0.3 on 2024-04-01 08:29
import django.contrib.postgres.indexes
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("product", "0008_product_product_created_at_idx_and_more"),
        ("vat", "0003_vat_vat_vat_value_d54579_btree"),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="product",
            name="product_pro_hits_19ae26_btree",
        ),
        migrations.RemoveField(
            model_name="product",
            name="hits",
        ),
        migrations.AddField(
            model_name="product",
            name="view_count",
            field=models.PositiveBigIntegerField(default=0, verbose_name="View Count"),
        ),
        migrations.AddIndex(
            model_name="product",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["view_count"], name="product_pro_view_co_bf71fd_btree"
            ),
        ),
    ]



from unittest.mock import MagicMock, patch

import pytest
from django.test import override_settings

from user.factories.account import UserAccountFactory
from user.models.subscription import SubscriptionTopic, UserSubscription
from user.utils.subscription import (
    check_subscription_before_send,
    generate_unsubscribe_link,
    get_user_subscription_summary,
    send_newsletter,
    send_subscription_confirmation,
)


@pytest.mark.django_db
class TestSubscriptionUtils:
    def setup_method(self):
        self.user = UserAccountFactory()
        self.topic = SubscriptionTopic.objects.create(
            name="Test Newsletter",
            slug="test-newsletter",
            description="Test newsletter description",
            category="news",
        )

    def test_check_subscription_before_send_active_exists(self):
        UserSubscription.objects.create(
            user=self.user,
            topic=self.topic,
            status=UserSubscription.SubscriptionStatus.ACTIVE,
        )

        result = check_subscription_before_send(self.user, self.topic.slug)

        assert result is True

    def test_check_subscription_before_send_no_active(self):
        UserSubscription.objects.create(
            user=self.user,
            topic=self.topic,
            status=UserSubscription.SubscriptionStatus.PENDING,
        )

        result = check_subscription_before_send(self.user, self.topic.slug)

        assert result is False

    def test_check_subscription_before_send_no_subscription(self):
        result = check_subscription_before_send(self.user, self.topic.slug)

        assert result is False

    @override_settings(
        SITE_NAME="Test Site",
        SUPPORT_EMAIL="support@test.com",
        DEFAULT_FROM_EMAIL="noreply@test.com",
    )
    @patch("user.utils.subscription.render_to_string")
    @patch("user.utils.subscription.EmailMultiAlternatives")
    @patch("user.utils.subscription.check_subscription_before_send")
    def test_send_subscription_confirmation_success(
        self, mock_check, mock_email_class, mock_render
    ):
        mock_check.return_value = False
        mock_render.return_value = "<html>Test confirmation email</html>"
        mock_email = MagicMock()
        mock_email_class.return_value = mock_email

        subscription = UserSubscription.objects.create(
            user=self.user,
            topic=self.topic,
            status=UserSubscription.SubscriptionStatus.PENDING,
            confirmation_token="test-token-123",
        )

        result = send_subscription_confirmation(subscription, self.user)

        assert result is True
        mock_check.assert_called_once_with(
            user=self.user, topic_slug=self.topic.slug
        )
        mock_render.assert_called_once()
        mock_email_class.assert_called_once()
        mock_email.attach_alternative.assert_called_once()
        mock_email.send.assert_called_once()

    @patch("user.utils.subscription.check_subscription_before_send")
    def test_send_subscription_confirmation_already_active(self, mock_check):
        mock_check.return_value = True

        subscription = UserSubscription.objects.create(
            user=self.user,
            topic=self.topic,
            status=UserSubscription.SubscriptionStatus.PENDING,
            confirmation_token="test-token-123",
        )

        result = send_subscription_confirmation(subscription, self.user)

        assert result is False

    def test_send_subscription_confirmation_not_pending(self):
        subscription = UserSubscription.objects.create(
            user=self.user,
            topic=self.topic,
            status=UserSubscription.SubscriptionStatus.ACTIVE,
            confirmation_token="test-token-123",
        )

        result = send_subscription_confirmation(subscription, self.user)

        assert result is False

    def test_send_subscription_confirmation_no_token(self):
        subscription = UserSubscription.objects.create(
            user=self.user,
            topic=self.topic,
            status=UserSubscription.SubscriptionStatus.PENDING,
            confirmation_token="",
        )

        result = send_subscription_confirmation(subscription, self.user)

        assert result is False

    @override_settings(
        DEFAULT_FROM_EMAIL="noreply@test.com",
    )
    @patch("user.utils.subscription.render_to_string")
    @patch("user.utils.subscription.EmailMultiAlternatives")
    @patch("user.utils.subscription.check_subscription_before_send")
    def test_send_subscription_confirmation_email_exception(
        self, mock_check, mock_email_class, mock_render
    ):
        mock_check.return_value = False
        mock_render.return_value = "<html>Test email</html>"
        mock_email = MagicMock()
        mock_email.send.side_effect = Exception("Email sending failed")
        mock_email_class.return_value = mock_email

        subscription = UserSubscription.objects.create(
            user=self.user,
            topic=self.topic,
            status=UserSubscription.SubscriptionStatus.PENDING,
            confirmation_token="test-token-123",
        )

        result = send_subscription_confirmation(subscription, self.user)

        assert result is False

    @override_settings(SITE_URL="https://test-site.com")
    @patch("user.utils.subscription.default_token_generator")
    @patch("user.utils.subscription.urlsafe_base64_encode")
    def test_generate_unsubscribe_link(self, mock_encode, mock_token_gen):
        mock_token_gen.make_token.return_value = "test-token"
        mock_encode.return_value = "encoded-uid"

        result = generate_unsubscribe_link(self.user, self.topic)

        expected_url = "https://test-site.com/unsubscribe/encoded-uid/test-token/test-newsletter/"
        assert result == expected_url
        mock_token_gen.make_token.assert_called_once_with(self.user)
        mock_encode.assert_called_once()

    @override_settings(SITE_URL="")
    @patch("user.utils.subscription.default_token_generator")
    @patch("user.utils.subscription.urlsafe_base64_encode")
    def test_generate_unsubscribe_link_no_site_url(
        self, mock_encode, mock_token_gen
    ):
        mock_token_gen.make_token.return_value = "test-token"
        mock_encode.return_value = "encoded-uid"

        result = generate_unsubscribe_link(self.user, self.topic)

        expected_url = "/unsubscribe/encoded-uid/test-token/test-newsletter/"
        assert result == expected_url

    @override_settings(
        DEFAULT_FROM_EMAIL="noreply@test.com", SITE_URL="https://test-site.com"
    )
    @patch("user.utils.subscription.render_to_string")
    @patch("user.utils.subscription.EmailMultiAlternatives")
    @patch("user.utils.subscription.generate_unsubscribe_link")
    def test_send_newsletter_success(
        self, mock_unsubscribe, mock_email_class, mock_render
    ):
        mock_render.return_value = "<html>Newsletter content</html>"
        mock_unsubscribe.return_value = "https://test.com/unsubscribe/123/"
        mock_email = MagicMock()
        mock_email_class.return_value = mock_email

        user2 = UserAccountFactory()
        UserSubscription.objects.create(
            user=self.user,
            topic=self.topic,
            status=UserSubscription.SubscriptionStatus.ACTIVE,
        )
        UserSubscription.objects.create(
            user=user2,
            topic=self.topic,
            status=UserSubscription.SubscriptionStatus.ACTIVE,
        )

        context = {"newsletter_title": "Test Newsletter"}

        result = send_newsletter(
            topic=self.topic,
            subject="Test Subject",
            template_name="newsletter/test.html",
            context=context,
            batch_size=50,
        )

        assert result["sent"] == 2
        assert result["failed"] == 0
        assert result["skipped"] == 0
        assert mock_email.send.call_count == 2

    @override_settings(
        DEFAULT_FROM_EMAIL="noreply@test.com", SITE_URL="https://test-site.com"
    )
    @patch("user.utils.subscription.render_to_string")
    @patch("user.utils.subscription.EmailMultiAlternatives")
    @patch("user.utils.subscription.generate_unsubscribe_link")
    def test_send_newsletter_with_inactive_user(
        self, mock_unsubscribe, mock_email_class, mock_render
    ):
        mock_render.return_value = "<html>Newsletter content</html>"
        mock_unsubscribe.return_value = "https://test.com/unsubscribe/123/"
        mock_email = MagicMock()
        mock_email_class.return_value = mock_email

        inactive_user = UserAccountFactory(is_active=False)
        UserSubscription.objects.create(
            user=inactive_user,
            topic=self.topic,
            status=UserSubscription.SubscriptionStatus.ACTIVE,
        )

        result = send_newsletter(
            topic=self.topic,
            subject="Test Subject",
            template_name="newsletter/test.html",
            context={},
        )

        assert result["sent"] == 0
        assert result["failed"] == 0
        assert result["skipped"] == 1
        mock_email.send.assert_not_called()

    @override_settings(
        DEFAULT_FROM_EMAIL="noreply@test.com", SITE_URL="https://test-site.com"
    )
    @patch("user.utils.subscription.render_to_string")
    @patch("user.utils.subscription.EmailMultiAlternatives")
    @patch("user.utils.subscription.generate_unsubscribe_link")
    def test_send_newsletter_email_exception(
        self, mock_unsubscribe, mock_email_class, mock_render
    ):
        mock_render.return_value = "<html>Newsletter content</html>"
        mock_unsubscribe.return_value = "https://test.com/unsubscribe/123/"
        mock_email = MagicMock()
        mock_email.send.side_effect = Exception("Email failed")
        mock_email_class.return_value = mock_email

        UserSubscription.objects.create(
            user=self.user,
            topic=self.topic,
            status=UserSubscription.SubscriptionStatus.ACTIVE,
        )

        result = send_newsletter(
            topic=self.topic,
            subject="Test Subject",
            template_name="newsletter/test.html",
            context={},
        )

        assert result["sent"] == 0
        assert result["failed"] == 1
        assert result["skipped"] == 0

    def test_send_newsletter_no_active_subscriptions(self):
        UserSubscription.objects.create(
            user=self.user,
            topic=self.topic,
            status=UserSubscription.SubscriptionStatus.PENDING,
        )

        result = send_newsletter(
            topic=self.topic,
            subject="Test Subject",
            template_name="newsletter/test.html",
            context={},
        )

        assert result["sent"] == 0
        assert result["failed"] == 0
        assert result["skipped"] == 0

    def test_get_user_subscription_summary_empty(self):
        result = get_user_subscription_summary(self.user)

        expected = {
            "total": 0,
            "active": 0,
            "pending": 0,
            "unsubscribed": 0,
            "by_category": {},
        }

        assert result == expected

    def test_get_user_subscription_summary_with_subscriptions(self):
        topic2 = SubscriptionTopic.objects.create(
            name="Tech News",
            slug="tech-news",
            description="Technology news",
            category="tech",
        )

        topic3 = SubscriptionTopic.objects.create(
            name="Sports Update",
            slug="sports-update",
            description="Sports updates",
            category="news",
        )

        UserSubscription.objects.create(
            user=self.user,
            topic=self.topic,
            status=UserSubscription.SubscriptionStatus.ACTIVE,
        )

        UserSubscription.objects.create(
            user=self.user,
            topic=topic2,
            status=UserSubscription.SubscriptionStatus.ACTIVE,
        )

        UserSubscription.objects.create(
            user=self.user,
            topic=topic3,
            status=UserSubscription.SubscriptionStatus.PENDING,
        )

        topic4 = SubscriptionTopic.objects.create(
            name="Another Newsletter",
            slug="another-newsletter",
            description="Another newsletter",
            category="news",
        )
        UserSubscription.objects.create(
            user=self.user,
            topic=topic4,
            status=UserSubscription.SubscriptionStatus.UNSUBSCRIBED,
        )

        result = get_user_subscription_summary(self.user)

        assert result["total"] == 4
        assert result["active"] == 2
        assert result["pending"] == 1
        assert result["unsubscribed"] == 1

        assert "news" in result["by_category"]
        assert "tech" in result["by_category"]

        assert result["by_category"]["news"]["total"] == 3
        assert result["by_category"]["news"]["active"] == 1

        assert result["by_category"]["tech"]["total"] == 1
        assert result["by_category"]["tech"]["active"] == 1

    def test_get_user_subscription_summary_different_user(self):
        other_user = UserAccountFactory()

        UserSubscription.objects.create(
            user=self.user,
            topic=self.topic,
            status=UserSubscription.SubscriptionStatus.ACTIVE,
        )

        result = get_user_subscription_summary(other_user)

        assert result["total"] == 0
        assert result["active"] == 0
        assert result["by_category"] == {}

    @patch("user.utils.subscription.logger")
    def test_send_subscription_confirmation_logs_warning_already_active(
        self, mock_logger
    ):
        with patch(
            "user.utils.subscription.check_subscription_before_send",
            return_value=True,
        ):
            subscription = UserSubscription.objects.create(
                user=self.user,
                topic=self.topic,
                status=UserSubscription.SubscriptionStatus.PENDING,
                confirmation_token="test-token",
            )

            send_subscription_confirmation(subscription, self.user)

            mock_logger.warning.assert_called_once()
            assert (
                "already active subscription"
                in mock_logger.warning.call_args[0][0]
            )

    @patch("user.utils.subscription.logger")
    def test_send_subscription_confirmation_logs_warning_not_pending(
        self, mock_logger
    ):
        subscription = UserSubscription.objects.create(
            user=self.user,
            topic=self.topic,
            status=UserSubscription.SubscriptionStatus.ACTIVE,
            confirmation_token="test-token",
        )

        send_subscription_confirmation(subscription, self.user)

        mock_logger.warning.assert_called_once()
        assert (
            "already active subscription" in mock_logger.warning.call_args[0][0]
        )

    @patch("user.utils.subscription.logger")
    def test_send_subscription_confirmation_logs_error_no_token(
        self, mock_logger
    ):
        subscription = UserSubscription.objects.create(
            user=self.user,
            topic=self.topic,
            status=UserSubscription.SubscriptionStatus.PENDING,
            confirmation_token="",
        )

        send_subscription_confirmation(subscription, self.user)

        mock_logger.error.assert_called_once()
        assert "No confirmation token" in mock_logger.error.call_args[0][0]

    @patch("user.utils.subscription.logger")
    def test_send_newsletter_logs_stats(self, mock_logger):
        UserSubscription.objects.create(
            user=self.user,
            topic=self.topic,
            status=UserSubscription.SubscriptionStatus.ACTIVE,
        )

        with (
            patch("user.utils.subscription.render_to_string"),
            patch("user.utils.subscription.EmailMultiAlternatives"),
            patch("user.utils.subscription.generate_unsubscribe_link"),
        ):
            send_newsletter(
                topic=self.topic,
                subject="Test",
                template_name="test.html",
                context={},
            )

            mock_logger.info.assert_called()
            log_message = mock_logger.info.call_args[0][0]
            assert "Newsletter sent for topic" in log_message
            assert "1 sent, 0 failed, 0 skipped" in log_message



from datetime import timedelta

from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django_filters import rest_framework as filters

from core.filters.camel_case_filters import (
    CamelCasePublishableTimeStampFilterSet,
)
from core.filters.core import UUIDFilterMixin
from product.enum.review import RateEnum, ReviewStatus
from product.models.review import ProductReview


class ProductReviewFilter(
    UUIDFilterMixin,
    CamelCasePublishableTimeStampFilterSet,
):
    id = filters.NumberFilter(
        field_name="id",
        help_text=_("Filter by review ID"),
    )
    user = filters.NumberFilter(
        field_name="user_id",
        help_text=_("Filter by user ID"),
    )
    user_id = filters.NumberFilter(
        field_name="user_id",
        help_text=_("Filter by user ID"),
    )
    product = filters.NumberFilter(
        field_name="product_id",
        help_text=_("Filter by product ID"),
    )
    product_id = filters.NumberFilter(
        field_name="product_id",
        help_text=_("Filter by product ID"),
    )

    status = filters.ChoiceFilter(
        field_name="status",
        choices=ReviewStatus.choices,
        help_text=_("Filter by review status"),
    )

    rate = filters.ChoiceFilter(
        field_name="rate",
        choices=RateEnum.choices,
        help_text=_("Filter by exact rating"),
    )
    rate_min = filters.NumberFilter(
        field_name="rate",
        lookup_expr="gte",
        help_text=_("Filter by minimum rating"),
    )
    rate_max = filters.NumberFilter(
        field_name="rate",
        lookup_expr="lte",
        help_text=_("Filter by maximum rating"),
    )
    min_rate = filters.NumberFilter(
        field_name="rate",
        lookup_expr="gte",
        help_text=_("Filter by minimum rating (alias)"),
    )
    max_rate = filters.NumberFilter(
        field_name="rate",
        lookup_expr="lte",
        help_text=_("Filter by maximum rating (alias)"),
    )

    comment = filters.CharFilter(
        field_name="translations__comment",
        lookup_expr="icontains",
        help_text=_("Filter by comment content (partial match)"),
    )
    has_comment = filters.BooleanFilter(
        method="filter_has_comment",
        help_text=_("Filter reviews that have/don't have comments"),
    )

    product_name = filters.CharFilter(
        field_name="product__translations__name",
        lookup_expr="icontains",
        help_text=_("Filter by product name (partial match)"),
    )
    product_category = filters.NumberFilter(
        field_name="product__category_id",
        help_text=_("Filter by product category ID"),
    )
    product_active = filters.BooleanFilter(
        field_name="product__active",
        help_text=_("Filter by product active status"),
    )
    user_email = filters.CharFilter(
        field_name="user__email",
        lookup_expr="icontains",
        help_text=_("Filter by user email (partial match)"),
    )
    user_first_name = filters.CharFilter(
        field_name="user__first_name",
        lookup_expr="icontains",
        help_text=_("Filter by user first name (partial match)"),
    )
    user_last_name = filters.CharFilter(
        field_name="user__last_name",
        lookup_expr="icontains",
        help_text=_("Filter by user last name (partial match)"),
    )

    recent_days = filters.NumberFilter(
        method="filter_recent_days",
        help_text=_("Filter reviews from the last N days"),
    )
    published_recent_days = filters.NumberFilter(
        method="filter_published_recent_days",
        help_text=_("Filter reviews published in the last N days"),
    )

    verified_purchase = filters.BooleanFilter(
        method="filter_verified_purchase",
        help_text=_(
            "Filter reviews from verified purchases (if order system exists)"
        ),
    )
    helpful_votes_min = filters.NumberFilter(
        method="filter_helpful_votes_min",
        help_text=_(
            "Filter by minimum helpful votes (if voting system exists)"
        ),
    )

    user_review_count_min = filters.NumberFilter(
        method="filter_user_review_count_min",
        help_text=_("Filter users with minimum number of reviews"),
    )
    product_avg_rating_min = filters.NumberFilter(
        method="filter_product_avg_rating_min",
        help_text=_("Filter products with minimum average rating"),
    )
    product_avg_rating_max = filters.NumberFilter(
        method="filter_product_avg_rating_max",
        help_text=_("Filter products with maximum average rating"),
    )

    class Meta:
        model = ProductReview
        fields = {
            "id": ["exact"],
            "user": ["exact"],
            "product": ["exact"],
            "status": ["exact"],
            "rate": ["exact", "gte", "lte"],
            "is_published": ["exact"],
            "created_at": ["gte", "lte", "date"],
            "updated_at": ["gte", "lte", "date"],
            "published_at": ["gte", "lte", "date"],
            "uuid": ["exact"],
        }

    def filter_has_comment(self, queryset, name, value):
        """Filter reviews that have/don't have comments"""
        if value is True:
            return queryset.exclude(
                models.Q(translations__comment__isnull=True)
                | models.Q(translations__comment__exact="")
            ).distinct()
        elif value is False:
            return queryset.filter(
                models.Q(translations__comment__isnull=True)
                | models.Q(translations__comment__exact="")
            ).distinct()
        return queryset

    def filter_recent_days(self, queryset, name, value):
        """Filter reviews from the last N days"""
        if value and value > 0:
            cutoff_date = timezone.now() - timedelta(days=int(value))
            return queryset.filter(created_at__gte=cutoff_date)
        return queryset

    def filter_published_recent_days(self, queryset, name, value):
        """Filter reviews published in the last N days"""
        if value and value > 0:
            cutoff_date = timezone.now() - timedelta(days=int(value))
            return queryset.filter(
                published_at__gte=cutoff_date, published_at__isnull=False
            )
        return queryset

    def filter_verified_purchase(self, queryset, name, value):
        """Filter reviews from verified purchases (placeholder for order system)"""
        # This would need to be implemented when order system exists
        # For now, just return the queryset as-is
        # In a real system, this might check:
        # return queryset.filter(user__orders__products=models.F('product'))
        return queryset

    def filter_helpful_votes_min(self, queryset, name, value):
        """Filter by minimum helpful votes (placeholder for voting system)"""
        # This would need to be implemented when review voting system exists
        # For now, just return the queryset as-is
        return queryset

    def filter_user_review_count_min(self, queryset, name, value):
        """Filter users with minimum number of reviews"""
        if value and value > 0:
            from django.db.models import Count

            users_with_min_reviews = (
                ProductReview.objects.values("user_id")
                .annotate(review_count=Count("id"))
                .filter(review_count__gte=value)
                .values_list("user_id", flat=True)
            )

            return queryset.filter(user_id__in=users_with_min_reviews)
        return queryset

    def filter_product_avg_rating_min(self, queryset, name, value):
        """Filter products with minimum average rating"""
        if value and value > 0:
            from django.db.models import Avg

            products_with_min_avg = (
                ProductReview.objects.values("product_id")
                .annotate(avg_rating=Avg("rate"))
                .filter(avg_rating__gte=value)
                .values_list("product_id", flat=True)
            )

            return queryset.filter(product_id__in=products_with_min_avg)
        return queryset

    def filter_product_avg_rating_max(self, queryset, name, value):
        """Filter products with maximum average rating"""
        if value and value > 0:
            from django.db.models import Avg

            products_with_max_avg = (
                ProductReview.objects.values("product_id")
                .annotate(avg_rating=Avg("rate"))
                .filter(avg_rating__lte=value)
                .values_list("product_id", flat=True)
            )

            return queryset.filter(product_id__in=products_with_max_avg)
        return queryset



from django.utils.translation import gettext_lazy as _
from django_filters import rest_framework as filters

from tag.models.tagged_item import TaggedItem
from core.filters.camel_case_filters import CamelCaseTimeStampFilterSet
from core.filters.core import UUIDFilterMixin


class TaggedItemFilter(UUIDFilterMixin, CamelCaseTimeStampFilterSet):
    tag = filters.NumberFilter(
        field_name="tag__id",
        help_text=_("Filter by specific tag ID"),
    )
    tag__label = filters.CharFilter(
        field_name="tag__translations__label",
        lookup_expr="icontains",
        help_text=_("Filter by tag label (partial match)"),
    )
    tag__active = filters.BooleanFilter(
        field_name="tag__active",
        help_text=_("Filter by tag active status"),
    )

    content_type = filters.CharFilter(
        field_name="content_type__model",
        help_text=_("Filter by content type model name"),
    )
    content_type__app_label = filters.CharFilter(
        field_name="content_type__app_label",
        help_text=_("Filter by content type app label"),
    )
    object_id = filters.NumberFilter(
        field_name="object_id",
        help_text=_("Filter by object ID"),
    )

    object_id__in = filters.BaseInFilter(
        field_name="object_id",
        help_text=_("Filter by multiple object IDs (comma-separated)"),
    )

    class Meta:
        model = TaggedItem
        fields = {
            "created_at": ["gte", "lte", "date"],
            "updated_at": ["gte", "lte", "date"],
            "uuid": ["exact"],
            "id": ["exact", "in"],
            "tag": ["exact"],
            "object_id": ["exact", "in"],
        }



from __future__ import annotations

from rest_framework import serializers

from blog.models.post import BlogPostTranslation
from product.models.product import ProductTranslation


class BlogPostTranslationSerializer(
    serializers.ModelSerializer[BlogPostTranslation]
):
    slug = serializers.SerializerMethodField()
    main_image_path = serializers.SerializerMethodField()
    matches_position = serializers.SerializerMethodField()
    ranking_score = serializers.SerializerMethodField()
    formatted = serializers.SerializerMethodField()
    content_type = serializers.SerializerMethodField()

    class Meta:
        model = BlogPostTranslation
        fields = (
            "id",
            "slug",
            "language_code",
            "title",
            "subtitle",
            "body",
            "master",
            "main_image_path",
            "matches_position",
            "ranking_score",
            "formatted",
            "content_type",
        )

    def get_slug(self, obj):
        return obj.master.slug if obj.master else ""

    def get_main_image_path(self, obj):
        return obj.master.main_image_path if obj.master else ""

    def get_matches_position(self, obj):
        return self.context.get("_matchesPosition", {})

    def get_ranking_score(self, obj):
        return self.context.get("_rankingScore", None)

    def get_formatted(self, obj):
        return self.context.get("_formatted", {})

    def get_content_type(self, obj):
        return "blog_post"


class ProductTranslationSerializer(
    serializers.ModelSerializer[ProductTranslation]
):
    slug = serializers.SerializerMethodField()
    main_image_path = serializers.SerializerMethodField()
    matches_position = serializers.SerializerMethodField()
    ranking_score = serializers.SerializerMethodField()
    formatted = serializers.SerializerMethodField()
    content_type = serializers.SerializerMethodField()

    class Meta:
        model = ProductTranslation
        fields = (
            "id",
            "slug",
            "language_code",
            "name",
            "description",
            "master",
            "main_image_path",
            "matches_position",
            "ranking_score",
            "formatted",
            "content_type",
        )

    def get_slug(self, obj):
        return obj.master.slug if obj.master else ""

    def get_main_image_path(self, obj):
        return obj.master.main_image_path if obj.master else ""

    def get_matches_position(self, obj):
        return self.context.get("_matchesPosition", {})

    def get_ranking_score(self, obj):
        return self.context.get("_rankingScore", None)

    def get_formatted(self, obj):
        return self.context.get("_formatted", {})

    def get_content_type(self, obj):
        return "product"


class BlogPostMeiliSearchResultSerializer(serializers.Serializer):
    id = serializers.IntegerField()
    language_code = serializers.CharField()
    title = serializers.CharField()
    subtitle = serializers.CharField()
    body = serializers.CharField()
    master = serializers.IntegerField()
    slug = serializers.CharField()
    main_image_path = serializers.CharField()
    matches_position = serializers.JSONField()
    ranking_score = serializers.FloatField(allow_null=True)
    formatted = serializers.JSONField()
    content_type = serializers.CharField()


class ProductMeiliSearchResultSerializer(serializers.Serializer):
    id = serializers.IntegerField()
    language_code = serializers.CharField()
    name = serializers.CharField()
    description = serializers.CharField()
    master = serializers.IntegerField()
    slug = serializers.CharField()
    main_image_path = serializers.CharField()
    matches_position = serializers.JSONField()
    ranking_score = serializers.FloatField(allow_null=True)
    formatted = serializers.JSONField()
    content_type = serializers.CharField()


class BlogPostMeiliSearchResponseSerializer(serializers.Serializer):
    limit = serializers.IntegerField()
    offset = serializers.IntegerField()
    estimated_total_hits = serializers.IntegerField()
    results = BlogPostMeiliSearchResultSerializer(many=True)


class ProductMeiliSearchResponseSerializer(serializers.Serializer):
    limit = serializers.IntegerField()
    offset = serializers.IntegerField()
    estimated_total_hits = serializers.IntegerField()
    results = ProductMeiliSearchResultSerializer(many=True)
